from ..player import PlayerId  # type: ignore
from ..animals import Animal  # type: ignore
from .. import Money, Value  # type: ignore

class NoAction:
    class Ok(NoAction): ...

class InitialTrade:
    opponent: PlayerId
    animal: Animal
    animal_count: int
    amount: list[Money]

    def __init__(
        self, opponent: PlayerId, animal: Animal, animal_count: int, amount: list[Money]
    ) -> None: ...

class PlayerTurnDecision:
    class Draw(PlayerTurnDecision): ...

    class Trade(PlayerTurnDecision):
        initial_trade: InitialTrade
        def __init__(self, initial_trade: InitialTrade) -> None: ...
        __match_args__ = ("initial_trade",)

class AuctionDecision:
    class Buy(AuctionDecision): ...
    class Sell(AuctionDecision): ...

class TradeOpponentDecision:
    class Accept(TradeOpponentDecision): ...

    class CounterOffer(TradeOpponentDecision):
        money_list: list[Money]
        def __init__(self, money_list: list[Money]) -> None: ...
        __match_args__ = ("money_list",)

class SendMoney:
    class WasBluff(SendMoney):
        def __init__(self) -> None: ...

    class Amount(SendMoney):
        money_list: list[Money]
        def __init__(self, money_list: list[Money]) -> None: ...
        __match_args__ = ("money_list",)

class Bidding:
    class Pass(Bidding):
        def __init__(self) -> None: ...

    class Bid(Bidding):
        value: Value
        def __init__(self, value: Value) -> None: ...



from abc import ABC, abstractmethod
from ..messages import actions, game_updates, message_protocol
from . import PlayerId
from .. import Value

class PlayerActions(ABC):
    @abstractmethod
    def _draw_or_trade(self) -> actions.PlayerTurnDecision: ...
    @abstractmethod
    def _trade(self) -> actions.InitialTrade: ...
    @abstractmethod
    def _provide_bidding(self, state: game_updates.AuctionRound) -> actions.Bidding: ...
    @abstractmethod
    def _buy_or_sell(
        self, state: game_updates.AuctionRound
    ) -> actions.AuctionDecision: ...
    @abstractmethod
    def _send_money_to_player(
        self, player: PlayerId, amount: Value
    ) -> actions.SendMoney: ...
    @abstractmethod
    def _respond_to_trade(
        self, offer: game_updates.TradeOffer
    ) -> actions.TradeOpponentDecision: ...
    @abstractmethod
    def _receive_game_update(
        self, update: game_updates.GameUpdate
    ) -> actions.NoAction: ...



from .player.player_actions import PlayerActions  # type: ignore

class Client:
    def __init__(
        self,
        name: str,
        token: str,
        bot: PlayerActions,
        base_url: str,
        raise_faulty_action_warning: bool,
    ): ...
    async def register(self): ...
    async def play_one_round(self, game_type_url: str): ...



from . import messages as messages  # type: ignore
from . import animals as animals  # type: ignore
from . import client as client  # type: ignore
from . import player as player  # type: ignore

from typing import TypeAlias

Money: TypeAlias = int
Value: TypeAlias = int
Points: TypeAlias = int

__all__ = [
    "player",
    "messages",
    "animals",
    "client",
    "player",
]



from . import Value  # type: ignore

class Animal:
    value: Value
    def __init__(self, value: Value) -> None: ...

class AnimalSet:
    animal: Animal
    inflation: list[Value]

    def __init__(
        self,
        value: Value,
        inflation: list[Value],
    ) -> None: ...



from .. import Money, Value

class Wallet:
    bank_notes: dict[Money, int]

    def __init__(self, bank_notes: dict[Money, int]) -> None: ...
    def withdraw_py(self, amount: list[Money]): ...
    def deposit_py(self, amount: list[Money]): ...
    def total_money_py(self) -> Value: ...
    def propose_bill_combinations_py(
        self, amount: Value, also_suggest_smaller_values: bool
    ) -> list[tuple[Value, list[Money]]]: ...
    def can_afford_py(self, payment_amount: list[Money]) -> Affordability: ...

class Affordability:
    class Exact(Affordability):
        money_list: list[Money]
        def __init__(self, money_list: list[Money]) -> None: ...
        __match_args__ = ("money_list",)

    class Alternative(Affordability):
        money_list: list[Money]
        def __init__(self, money_list: list[Money]) -> None: ...
        __match_args__ = ("money_list",)

    class CannotAfford(Affordability): ...



from ..messages import actions
from .player_actions import PlayerActions
from ..player import PlayerId

class RandomPlayerActions(PlayerActions):
    def __init__(self, id: PlayerId, seed: int): ...
    def _draw_or_trade(self) -> actions.PlayerTurnDecision: ...
    def _trade(self) -> actions.InitialTrade: ...
    def _provide_bidding(self, state) -> actions.Bidding: ...
    def _buy_or_sell(self, state) -> actions.AuctionDecision: ...
    def _send_money_to_player(self, player, amount) -> actions.SendMoney: ...
    def _respond_to_trade(self, offer) -> actions.TradeOpponentDecision: ...
    def _receive_game_update(self, update) -> actions.NoAction: ...
